"""
PyPrestaScan - Scanner SEO professionale per PrestaShop
"""

__version__ = "1.7.1"
__author__ = "Andrea Piani"
__email__ = "info@andreapiani.com"
__all__ = ["__version__", "__author__", "__email__"]