
        .kpi-card { 
            background: linear-gradient(135deg, #667eea 0%, #764ba2 100%);
            color: white;
            border-radius: 8px;
            padding: 20px;
            margin: 10px;
        }
        .score-excellent { color: #28a745; }
        .score-good { color: #17a2b8; }
        .score-warning { color: #ffc107; }
        .score-poor { color: #dc3545; }
        .issue-critical { background-color: #f8d7da; }
        .issue-warning { background-color: #fff3cd; }
        .issue-info { background-color: #d1ecf1; }
        